package com.sun.media.sound;

import java.util.Comparator;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

public final class SoftPerformer
{
    static ModelConnectionBlock[] defaultconnections;
    public int keyFrom;
    public int keyTo;
    public int velFrom;
    public int velTo;
    public int exclusiveClass;
    public boolean selfNonExclusive;
    public boolean forcedVelocity;
    public boolean forcedKeynumber;
    public ModelPerformer performer;
    public ModelConnectionBlock[] connections;
    public ModelOscillator[] oscillators;
    public Map<Integer, int[]> midi_rpn_connections;
    public Map<Integer, int[]> midi_nrpn_connections;
    public int[][] midi_ctrl_connections;
    public int[][] midi_connections;
    public int[] ctrl_connections;
    private List<Integer> ctrl_connections_list;
    private static SoftPerformer.KeySortComparator keySortComparator;
    
    private String extractKeys(ModelConnectionBlock modelConnectionBlock) {
        StringBuffer sb = new StringBuffer();
        if (modelConnectionBlock.getSources() != null) {
            sb.append("[");
            ModelSource[] sources = modelConnectionBlock.getSources();
            ModelSource[] a = new ModelSource[sources.length];
            for (int i = 0; i < sources.length; ++i) {
                a[i] = sources[i];
            }
            Arrays.sort(a, SoftPerformer.keySortComparator);
            for (int j = 0; j < sources.length; ++j) {
                sb.append(sources[j].getIdentifier());
                sb.append(";");
            }
            sb.append("]");
        }
        sb.append(";");
        if (modelConnectionBlock.getDestination() != null) {
            sb.append(modelConnectionBlock.getDestination().getIdentifier());
        }
        sb.append(";");
        return sb.toString();
    }
    
    private void processSource(ModelSource modelSource, int i) {
        String object = modelSource.getIdentifier().getObject();
        if (object.equals("midi_cc")) {
            this.processMidiControlSource(modelSource, i);
        }
        else if (object.equals("midi_rpn")) {
            this.processMidiRpnSource(modelSource, i);
        }
        else if (object.equals("midi_nrpn")) {
            this.processMidiNrpnSource(modelSource, i);
        }
        else if (object.equals("midi")) {
            this.processMidiSource(modelSource, i);
        }
        else if (object.equals("noteon")) {
            this.processNoteOnSource(modelSource, i);
        }
        else {
            if (object.equals("osc")) {
                return;
            }
            if (object.equals("mixer")) {
                return;
            }
            this.ctrl_connections_list.add(Integer.valueOf(i));
        }
    }
    
    private void processMidiControlSource(ModelSource modelSource, int n) {
        String variable = modelSource.getIdentifier().getVariable();
        if (variable == null) {
            return;
        }
        int int1 = Integer.parseInt(variable);
        if (this.midi_ctrl_connections[int1] == null) {
            this.midi_ctrl_connections[int1] = new int[] { n };
        }
        else {
            int[] array = this.midi_ctrl_connections[int1];
            int[] array2 = new int[array.length + 1];
            for (int i = 0; i < array.length; ++i) {
                array2[i] = array[i];
            }
            array2[array2.length - 1] = n;
            this.midi_ctrl_connections[int1] = array2;
        }
        return;
    }
    
    private void processNoteOnSource(ModelSource modelSource, int n) {
        String variable = modelSource.getIdentifier().getVariable();
        int n2 = -1;
        if (variable.equals("on")) {
            n2 = 3;
        }
        if (variable.equals("keynumber")) {
            n2 = 4;
        }
        if (n2 == -1) {
            return;
        }
        if (this.midi_connections[n2] == null) {
            this.midi_connections[n2] = new int[] { n };
        }
        else {
            int[] array = this.midi_connections[n2];
            int[] array2 = new int[array.length + 1];
            for (int i = 0; i < array.length; ++i) {
                array2[i] = array[i];
            }
            array2[array2.length - 1] = n;
            this.midi_connections[n2] = array2;
        }
        return;
    }
    
    private void processMidiSource(ModelSource modelSource, int n) {
        String variable = modelSource.getIdentifier().getVariable();
        int n2 = -1;
        if (variable.equals("pitch")) {
            n2 = 0;
        }
        if (variable.equals("channel_pressure")) {
            n2 = 1;
        }
        if (variable.equals("poly_pressure")) {
            n2 = 2;
        }
        if (n2 == -1) {
            return;
        }
        if (this.midi_connections[n2] == null) {
            this.midi_connections[n2] = new int[] { n };
        }
        else {
            int[] array = this.midi_connections[n2];
            int[] array2 = new int[array.length + 1];
            for (int i = 0; i < array.length; ++i) {
                array2[i] = array[i];
            }
            array2[array2.length - 1] = n;
            this.midi_connections[n2] = array2;
        }
        return;
    }
    
    private void processMidiRpnSource(ModelSource modelSource, int n) {
        String variable = modelSource.getIdentifier().getVariable();
        if (variable == null) {
            return;
        }
        int int1 = Integer.parseInt(variable);
        if (this.midi_rpn_connections.get(Integer.valueOf(int1)) == null) {
            this.midi_rpn_connections.put(Integer.valueOf(int1), new int[] { n });
        }
        else {
            int[] array = (int[])this.midi_rpn_connections.get(Integer.valueOf(int1));
            int[] array2 = new int[array.length + 1];
            for (int i = 0; i < array.length; ++i) {
                array2[i] = array[i];
            }
            array2[array2.length - 1] = n;
            this.midi_rpn_connections.put(Integer.valueOf(int1), array2);
        }
        return;
    }
    
    private void processMidiNrpnSource(ModelSource modelSource, int n) {
        String variable = modelSource.getIdentifier().getVariable();
        if (variable == null) {
            return;
        }
        int int1 = Integer.parseInt(variable);
        if (this.midi_nrpn_connections.get(Integer.valueOf(int1)) == null) {
            this.midi_nrpn_connections.put(Integer.valueOf(int1), new int[] { n });
        }
        else {
            int[] array = (int[])this.midi_nrpn_connections.get(Integer.valueOf(int1));
            int[] array2 = new int[array.length + 1];
            for (int i = 0; i < array.length; ++i) {
                array2[i] = array[i];
            }
            array2[array2.length - 1] = n;
            this.midi_nrpn_connections.put(Integer.valueOf(int1), array2);
        }
        return;
    }
    
    public SoftPerformer(ModelPerformer performer) {
        this.keyFrom = 0;
        this.keyTo = 127;
        this.velFrom = 0;
        this.velTo = 127;
        this.exclusiveClass = 0;
        this.selfNonExclusive = false;
        this.forcedVelocity = false;
        this.forcedKeynumber = false;
        this.midi_rpn_connections = new HashMap();
        this.midi_nrpn_connections = new HashMap();
        this.ctrl_connections_list = new ArrayList();
        this.performer = performer;
        this.keyFrom = performer.getKeyFrom();
        this.keyTo = performer.getKeyTo();
        this.velFrom = performer.getVelFrom();
        this.velTo = performer.getVelTo();
        this.exclusiveClass = performer.getExclusiveClass();
        this.selfNonExclusive = performer.isSelfNonExclusive();
        HashMap<String, ModelConnectionBlock> hashMap = new HashMap();
        ArrayList list = new ArrayList();
        list.addAll(performer.getConnectionBlocks());
        if (performer.isDefaultConnectionsEnabled()) {
            boolean n = false;
            for (int i = 0; i < list.size(); ++i) {
                ModelConnectionBlock modelConnectionBlock = (ModelConnectionBlock)list.get(i);
                ModelSource[] sources = modelConnectionBlock.getSources();
                ModelDestination destination = modelConnectionBlock.getDestination();
                boolean n2 = false;
                if (destination != null && sources != null && sources.length > 1) {
                    int j = 0;
                    while (j < sources.length) {
                        if (sources[j].getIdentifier().getObject().equals("midi_cc") && sources[j].getIdentifier().getVariable().equals("1")) {
                            n2 = true;
                            n = true;
                            break;
                        }
                        ++j;
                    }
                }
                if (!(n2 == 0)) {
                    ModelConnectionBlock o = new ModelConnectionBlock();
                    o.setSources(modelConnectionBlock.getSources());
                    o.setDestination(modelConnectionBlock.getDestination());
                    o.addSource(new ModelSource(new ModelIdentifier("midi_rpn", "5")));
                    o.setScale(modelConnectionBlock.getScale() * 256.0);
                    list.set(i, o);
                }
            }
            if (n == 0) {
                ModelConnectionBlock o2 = new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_LFO1, false, true, 0), new ModelSource(new ModelIdentifier("midi_cc", "1", 0), false, false, 0), 50.0, new ModelDestination(ModelDestination.DESTINATION_PITCH));
                o2.addSource(new ModelSource(new ModelIdentifier("midi_rpn", "5")));
                o2.setScale(o2.getScale() * 256.0);
                list.add(o2);
            }
            boolean n3 = false;
            boolean n4 = false;
            ModelConnectionBlock modelConnectionBlock2 = null;
            int n5 = 0;
            for (ModelConnectionBlock modelConnectionBlock3 : list) {
                ModelSource[] sources2 = modelConnectionBlock3.getSources();
                if (modelConnectionBlock3.getDestination() != null && sources2 != null) {
                    for (int k = 0; k < sources2.length; ++k) {
                        ModelIdentifier identifier = sources2[k].getIdentifier();
                        if (identifier.getObject().equals("midi_cc") && identifier.getVariable().equals("1")) {
                            modelConnectionBlock2 = modelConnectionBlock3;
                            n5 = k;
                        }
                        if (identifier.getObject().equals("midi")) {
                            if (identifier.getVariable().equals("channel_pressure")) {
                                n3 = true;
                            }
                            if (identifier.getVariable().equals("poly_pressure")) {
                                n4 = true;
                            }
                        }
                    }
                }
                else {
                    continue;
                }
            }
            if (modelConnectionBlock2 != null) {
                if (n3 == 0) {
                    ModelConnectionBlock o3 = new ModelConnectionBlock();
                    o3.setDestination(modelConnectionBlock2.getDestination());
                    o3.setScale(modelConnectionBlock2.getScale());
                    ModelSource[] sources3 = modelConnectionBlock2.getSources();
                    ModelSource[] sources4 = new ModelSource[sources3.length];
                    for (int l = 0; l < sources4.length; ++l) {
                        sources4[l] = sources3[l];
                    }
                    sources4[n5] = new ModelSource(new ModelIdentifier("midi", "channel_pressure"));
                    o3.setSources(sources4);
                    hashMap.put(this.extractKeys(o3), o3);
                }
                if (n4 == 0) {
                    ModelConnectionBlock o4 = new ModelConnectionBlock();
                    o4.setDestination(modelConnectionBlock2.getDestination());
                    o4.setScale(modelConnectionBlock2.getScale());
                    ModelSource[] sources5 = modelConnectionBlock2.getSources();
                    ModelSource[] sources6 = new ModelSource[sources5.length];
                    for (int n6 = 0; n6 < sources6.length; ++n6) {
                        sources6[n6] = sources5[n6];
                    }
                    sources6[n5] = new ModelSource(new ModelIdentifier("midi", "poly_pressure"));
                    o4.setSources(sources6);
                    hashMap.put(this.extractKeys(o4), o4);
                }
            }
            ModelConnectionBlock modelConnectionBlock4 = null;
            for (ModelConnectionBlock modelConnectionBlock5 : list) {
                ModelSource[] sources7 = modelConnectionBlock5.getSources();
                if (sources7.length != 0 && sources7[0].getIdentifier().getObject().equals("lfo") && modelConnectionBlock5.getDestination().getIdentifier().equals(ModelDestination.DESTINATION_PITCH)) {
                    if (modelConnectionBlock4 == null) {
                        modelConnectionBlock4 = modelConnectionBlock5;
                    }
                    else if (modelConnectionBlock4.getSources().length > sources7.length) {
                        modelConnectionBlock4 = modelConnectionBlock5;
                    }
                    else if (modelConnectionBlock4.getSources()[0].getIdentifier().getInstance() < 1 && modelConnectionBlock4.getSources()[0].getIdentifier().getInstance() > sources7[0].getIdentifier().getInstance()) {
                        modelConnectionBlock4 = modelConnectionBlock5;
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
            int instance = 1;
            if (modelConnectionBlock4 != null) {
                instance = modelConnectionBlock4.getSources()[0].getIdentifier().getInstance();
            }
            ModelConnectionBlock o5 = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "78"), false, true, 0), 2000.0, new ModelDestination(new ModelIdentifier("lfo", "delay2", instance)));
            hashMap.put(this.extractKeys(o5), o5);
            ModelConnectionBlock o6 = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("lfo", instance)), new ModelSource(new ModelIdentifier("midi_cc", "77"), new ModelTransform() {
                double s = scale;
                
                @Override
                public double transform(double n) {
                    n = n * 2.0 - 1.0;
                    n *= 600.0;
                    if (this.s == 0.0) {
                        return n;
                    }
                    if (this.s > 0.0) {
                        if (n < -this.s) {
                            n = -this.s;
                        }
                        return n;
                    }
                    else {
                        if (n < this.s) {
                            n = -this.s;
                        }
                        return -n;
                    }
                }
            }), new ModelDestination(ModelDestination.DESTINATION_PITCH));
            hashMap.put(this.extractKeys(o6), o6);
            ModelConnectionBlock o7 = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "76"), false, true, 0), 2400.0, new ModelDestination(new ModelIdentifier("lfo", "freq", instance)));
            hashMap.put(this.extractKeys(o7), o7);
        }
        if (performer.isDefaultConnectionsEnabled()) {
            for (ModelConnectionBlock modelConnectionBlock6 : SoftPerformer.defaultconnections) {
                hashMap.put(this.extractKeys(modelConnectionBlock6), modelConnectionBlock6);
            }
        }
        for (ModelConnectionBlock modelConnectionBlock7 : list) {
            hashMap.put(this.extractKeys(modelConnectionBlock7), modelConnectionBlock7);
        }
        ArrayList o8 = new ArrayList();
        this.midi_ctrl_connections = new int[128][];
        for (int n8 = 0; n8 < this.midi_ctrl_connections.length; ++n8) {
            this.midi_ctrl_connections[n8] = null;
        }
        this.midi_connections = new int[5][];
        for (int n9 = 0; n9 < this.midi_connections.length; ++n9) {
            this.midi_connections[n9] = null;
        }
        int n10 = 0;
        int n11 = 0;
        for (ModelConnectionBlock modelConnectionBlock8 : hashMap.values()) {
            if (modelConnectionBlock8.getDestination() != null) {
                ModelIdentifier identifier2 = modelConnectionBlock8.getDestination().getIdentifier();
                if (identifier2.getObject().equals("noteon")) {
                    n11 = 1;
                    if (identifier2.getVariable().equals("keynumber")) {
                        this.forcedKeynumber = true;
                    }
                    if (identifier2.getVariable().equals("velocity")) {
                        this.forcedVelocity = true;
                    }
                }
            }
            if (n11 != 0) {
                o8.add(0, modelConnectionBlock8);
                n11 = 0;
            }
            else {
                o8.add(modelConnectionBlock8);
            }
        }
        for (ModelConnectionBlock modelConnectionBlock9 : o8) {
            if (modelConnectionBlock9.getSources() != null) {
                ModelSource[] sources8 = modelConnectionBlock9.getSources();
                for (int n12 = 0; n12 < sources8.length; ++n12) {
                    this.processSource(sources8[n12], n10);
                }
            }
            ++n10;
        }
        o8.toArray(this.connections = new ModelConnectionBlock[o8.size()]);
        this.ctrl_connections = new int[this.ctrl_connections_list.size()];
        for (int n13 = 0; n13 < this.ctrl_connections.length; ++n13) {
            this.ctrl_connections[n13] = (int)this.ctrl_connections_list.get(n13);
        }
        this.oscillators = new ModelOscillator[performer.getOscillators().size()];
        performer.getOscillators().toArray(this.oscillators);
        for (ModelConnectionBlock modelConnectionBlock10 : o8) {
            if (modelConnectionBlock10.getDestination() != null && isUnnecessaryTransform(modelConnectionBlock10.getDestination().getTransform())) {
                modelConnectionBlock10.getDestination().setTransform(null);
            }
            if (modelConnectionBlock10.getSources() != null) {
                for (ModelSource modelSource : modelConnectionBlock10.getSources()) {
                    if (isUnnecessaryTransform(modelSource.getTransform())) {
                        modelSource.setTransform(null);
                    }
                }
            }
            else {
                continue;
            }
        }
    }
    
    private static boolean isUnnecessaryTransform(ModelTransform modelTransform) {
        if (modelTransform == null) {
            return false;
        }
        if (!(modelTransform instanceof ModelStandardTransform)) {
            return false;
        }
        ModelStandardTransform modelStandardTransform = (ModelStandardTransform)modelTransform;
        return !modelStandardTransform.getDirection() && !modelStandardTransform.getPolarity() && modelStandardTransform.getTransform() != 0 && false;
    }
    
    static {
        SoftPerformer.defaultconnections = new ModelConnectionBlock[42];
        int n = 0;
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("noteon", "on", 0), false, false, 0), 1.0, new ModelDestination(new ModelIdentifier("eg", "on", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("noteon", "on", 0), false, false, 0), 1.0, new ModelDestination(new ModelIdentifier("eg", "on", 1)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("eg", "active", 0), false, false, 0), 1.0, new ModelDestination(new ModelIdentifier("mixer", "active", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("eg", 0), true, false, 0), -960.0, new ModelDestination(new ModelIdentifier("mixer", "gain")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("noteon", "velocity"), true, false, 1), -960.0, new ModelDestination(new ModelIdentifier("mixer", "gain")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi", "pitch"), false, true, 0), new ModelSource(new ModelIdentifier("midi_rpn", "0"), new ModelTransform() {
            @Override
            public double transform(double n) {
                int n2 = (int)(n * 16384.0);
                return (double)((n2 >> 7) * 100 + (n2 & 0x7F));
            }
        }), new ModelDestination(new ModelIdentifier("osc", "pitch")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("noteon", "keynumber"), false, false, 0), 12800.0, new ModelDestination(new ModelIdentifier("osc", "pitch")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "7"), true, false, 1), -960.0, new ModelDestination(new ModelIdentifier("mixer", "gain")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "8"), false, false, 0), 1000.0, new ModelDestination(new ModelIdentifier("mixer", "balance")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "10"), false, false, 0), 1000.0, new ModelDestination(new ModelIdentifier("mixer", "pan")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "11"), true, false, 1), -960.0, new ModelDestination(new ModelIdentifier("mixer", "gain")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "91"), false, false, 0), 1000.0, new ModelDestination(new ModelIdentifier("mixer", "reverb")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "93"), false, false, 0), 1000.0, new ModelDestination(new ModelIdentifier("mixer", "chorus")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "71"), false, true, 0), 200.0, new ModelDestination(new ModelIdentifier("filter", "q")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "74"), false, true, 0), 9600.0, new ModelDestination(new ModelIdentifier("filter", "freq")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "72"), false, true, 0), 6000.0, new ModelDestination(new ModelIdentifier("eg", "release2")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "73"), false, true, 0), 2000.0, new ModelDestination(new ModelIdentifier("eg", "attack2")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "75"), false, true, 0), 6000.0, new ModelDestination(new ModelIdentifier("eg", "decay2")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "67"), false, false, 3), -50.0, new ModelDestination(ModelDestination.DESTINATION_GAIN));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_cc", "67"), false, false, 3), -2400.0, new ModelDestination(ModelDestination.DESTINATION_FILTER_FREQ));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_rpn", "1"), false, true, 0), 100.0, new ModelDestination(new ModelIdentifier("osc", "pitch")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("midi_rpn", "2"), false, true, 0), 12800.0, new ModelDestination(new ModelIdentifier("osc", "pitch")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("master", "fine_tuning"), false, true, 0), 100.0, new ModelDestination(new ModelIdentifier("osc", "pitch")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(new ModelSource(new ModelIdentifier("master", "coarse_tuning"), false, true, 0), 12800.0, new ModelDestination(new ModelIdentifier("osc", "pitch")));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(13500.0, new ModelDestination(new ModelIdentifier("filter", "freq", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("eg", "delay", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("eg", "attack", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("eg", "hold", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("eg", "decay", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(1000.0, new ModelDestination(new ModelIdentifier("eg", "sustain", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("eg", "release", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(1200.0 * Math.log(0.015) / Math.log(2.0), new ModelDestination(new ModelIdentifier("eg", "shutdown", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("eg", "delay", 1)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("eg", "attack", 1)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("eg", "hold", 1)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("eg", "decay", 1)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(1000.0, new ModelDestination(new ModelIdentifier("eg", "sustain", 1)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("eg", "release", 1)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(-8.51318, new ModelDestination(new ModelIdentifier("lfo", "freq", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("lfo", "delay", 0)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(-8.51318, new ModelDestination(new ModelIdentifier("lfo", "freq", 1)));
        SoftPerformer.defaultconnections[n++] = new ModelConnectionBlock(Double.NEGATIVE_INFINITY, new ModelDestination(new ModelIdentifier("lfo", "delay", 1)));
        SoftPerformer.keySortComparator = new SoftPerformer.KeySortComparator();
    }
    
    private static class KeySortComparator implements Comparator<ModelSource>
    {
        @Override
        public int compare(ModelSource modelSource, ModelSource modelSource2) {
            return modelSource.getIdentifier().toString().compareTo(modelSource2.getIdentifier().toString());
        }
    }
}
